/*-- Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA. --*/ 
/*-- All Rights Reserved.                                     --*/

/*****************************************************************************/
/* PAM_FILTER_SUMMARIZE_EVENT_DATA 
/*
/* PURPOSE: Wrapper around the PAM_DATAREDUCE MACRO.
/*
/* PARAMETERS: Global macro variables.
/*
/*    P_INPUT_DS=   Required input data set to be reduced.
/*                    <libref>.<data set name>.
/*****************************************************************************/

%macro pam_filter_summarize_event_data;
  
  %global P_MAX_OBSERVATIONS g_indsnsize p_x_varname ;
  
  /*** parameter validation ***/
  

  /* p_x_varname */  
  %if %nrquote(&p_x_varname) = %str() %then %do;
    %let p_x_varname=EVENT_DTTM;
  %end;
  
  
  %pam_datareduce(indsn = &p_input_ds,
                  xvar = &p_x_varname,
                  maxobs = &P_MAX_OBSERVATIONS,
                  indsnsizevar = g_indsnsize);
                  
%finish:
%mend;
/****************************************************************************/
/* PAM_DATAREDUCE  
/* Creates a copy of an input data set but reduces the amount of 
/*   observations.  The reduction is based on defining a set of buckets
/*   for a specified X variable in the input data set.  Only one 
/*   observation per bucket is placed into the output data set.
/*
/* PARAMETERS
/*    INDSN=    Required input data set.  The reduce code will write
/*                 the output to the same name data set - indsn.
/*    XVAR=     Required variable name whose values will determine the 
/*                 buckets to bucket the data.  The high and low values
/*                 for this variable are subtracted to get a range.  The
/*                 range is divided by the number of buckets to get the size
/*                 of each bucket.  XVAR is typically set to EVENT_DTTM.
/*                 The number of buckets is the MAXOBS= value.  
/*    MAXOBS=  Maximum number of observations in the output data set.  
/*                 This number also determines the number of buckets
/*                 because no more than one observation will be stored
/*                 in each bucket.  This parameter defaults to 10,000.
/*    INDSNSIZEVAR = 
/*              Optional name of a macro variable whose value will be 
/*                 set to the number of observations in the INDSN data set.
/*                 If this variable does not exist, it will be created
/*                 globally.
/**************************************************************************/
%macro pam_datareduce(indsn=, xvar=, xvarbegin=,  
                      maxobs=, indsnsizevar=g_indsnsize);
                      
 
%local defaultmaxobs;
%let defaultmaxobs=10000;

/*** parameter validation ***/

/* INDSN */
%if %sysfunc(exist(&indsn)) = 0 %then %do;
   %put ERROR: &sysmacroname must have a valid value for parameter INDSN;
   %goto finish;
%end;

/* XVAR */
%local xvarnum dsid;
%let dsid = %sysfunc(open(&indsn));
%let xvarnum = %sysfunc(varnum(&dsid, &xvar));
%let dsid = %sysfunc(close(&dsid));
%if &xvarnum <= 0 %then %do;
  %put ERROR: &sysmacroname must have a valid value for parameter XVAR;
  %goto finish;
%end;

/* MAXOBS */
%if %nrquote(&maxobs) = %str() %then %do;
  %put NOTE: &sysmacroname is using the default value of &defaultmaxobs for the MAXOBS parameter;
  %let maxobs = &defaultmaxobs;
%end;
 
%if %nrquote(&indsnsizevar) ne %str() %then %do;
  %if %sysfunc(nvalid(&indsnsizevar)) = 0 %then %do;
     %put ERROR: If supplied, the optional parameter INDSNSIZEVAR must contain a valid macro variable name.  The supplied value: &&indsnsizevar is not valid.;
     %goto finish;
  %end;
%end;

/*** end of parameter validation ***/

/* get the variables and other DS attributes*/
%local dsid nvars i varname varnames nobs;
%let dsid = %sysfunc(open(&indsn));
%let nvars = %sysfunc(attrn(&dsid, NVARS));
%let nobs = %sysfunc(attrn(&dsid, NOBS)); 
%let dsid = %sysfunc(close(&dsid));

/* if supplied, make sure the name is defined as a global macro variable and set its value to */
/* the number of observations in the input data set */
%if %nrquote(&indsnsizevar) ne %str( ) %then %do;
   %global &indsnsizevar;
   %let &indsnsizevar=&nobs;
%end; 

%if &nobs = 0 %then %do;
   %put WARNING: &sysmacroname:  Input data set &indsn has zero observations.;
%end;

%if &nobs <= &maxobs %then %do;
   %put NOTE: &sysmacroname:   Input data set contains less than &maxobs observations.;
   %put NOTE: &sysmacroname:   No data reduction was necessary.; 
   %put NOTE: &sysmacroname:   All data points will be plotted.;
   %goto finish;
%end;

%let dsid = %sysfunc(open(&indsn));
%do i=1 %to &nvars;
   %local varname&i;
   %let varname&i = %sysfunc(varname(&dsid, &i));
%end; 
%let dsid=%sysfunc(close(&dsid)); 
 

%if &nobs > &maxobs %then %do;
  data _null_;
    set &indsn(obs=1);
    call symputx('firstdt',event_dttm);
  run;

  data _null_;
    lastrec=&nobs;
    set &indsn point=lastrec;
    call symputx('lastdt',&xvar);
    output;
    stop;
  run;

  data &indsn(keep=&xvar event_value);
    set &indsn;
    retain intervalLength intervalBegin intervalEnd 0;
    intervalLength = %sysevalf((&lastdt - &firstdt)/&maxobs);

    if _n_ = 1 then do;
      intervalBegin = &firstdt + intervalLength;
      intervalEnd = intervalBegin + intervalLength;
      output;
    end;
    else if intervalBegin <= &xvar <= intervalEnd then do;
      intervalBegin = intervalEnd;
      intervalEnd = intervalBegin + intervalLength;
      output;
    end;
    else do;
      do i=1 to &maxobs;
        newBegin = intervalBegin + i*intervalLength;
        newEnd = intervalEnd + i*intervalLength;
        if newBegin <= &xvar <= newEnd then do;
          intervalBegin = newBegin; 
          intervalEnd = newEnd;
          output;
        end;
      end;
    end; 
  run;

%end;

%finish:
%mend;
